/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.sql.Connection;

import javax.validation.ConstraintValidator;

import org.databene.commons.ArrayFormat;
import org.databene.commons.ArrayUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.StringUtil;
import org.databene.commons.Validator;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.commons.iterator.TableRowIteratorProxy;
import org.databene.commons.validator.bean.BeanConstraintValidator;
import org.databene.jdbacl.ArrayResultSetIterator;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;

/**
 * {@link CheckStrategy} that uses a {@link Validator} to check database data on the client side.<br/><br/>
 * Created: 26.10.2011 20:58:03
 * @since 0.9.0
 * @author Volker Bergmann
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class ValidatorStrategy extends AbstractCheckStrategy {

	private Validator validator;
	private String tableName;
	private String expression;
	private String condition;
	
	private String reportColumnsSpec;
	private String[] reportColumns;
	
	private int checkedColumnIndex;
	private long defectCount;
	
	public ValidatorStrategy(Object validatorObject, String tableName, String expression, String condition, String reportColumnsSpec) {
		this(validatorObject, tableName, expression, condition, reportColumnsSpec, null);
	}
	
	public ValidatorStrategy(Object validatorObject, String tableName, String expression, String condition, String[] reportColumns) {
		this(validatorObject, tableName, expression, condition, null, reportColumns);
	}
	
	private ValidatorStrategy(Object validatorObject, String tableName, String expression, String condition, String reportColumnsSpec, String[] reportColumns) {
		if (validatorObject instanceof Validator)
			this.validator = (Validator) validatorObject;
		else if (validatorObject instanceof ConstraintValidator)
			this.validator = new BeanConstraintValidator((ConstraintValidator) validatorObject);
		else
			throw new ConfigurationError("Not a supported validator type: " + validatorObject.getClass().getName());
		this.tableName = tableName;
		this.expression = expression;
		this.condition = condition;
		this.reportColumnsSpec = reportColumnsSpec;
		this.reportColumns = reportColumns;
		this.defectCount = 0;
	}
	
	public String getDetails() {
		return validator.toString();
	}

	public Validator getValidator() {
		return validator;
	}

	public TableRowIterator perform(Connection connection, Database database) {
		DBTable table = database.getTable(tableName);
		if (reportColumns == null)
			reportColumns = parseReportColumns(reportColumnsSpec, table);
		this.checkedColumnIndex = StringUtil.indexOfIgnoreCase(expression, reportColumns);
		if (this.checkedColumnIndex < 0) {
			reportColumns = ArrayUtil.append(expression, reportColumns);
			this.checkedColumnIndex = reportColumns.length;
		}
		String query = "SELECT " + ArrayFormat.format(reportColumns) + " FROM " + table;
		if (!StringUtil.isEmpty(condition))
			query += " WHERE " + condition;
		ArrayResultSetIterator iterator = new ArrayResultSetIterator(connection, query);
		return new Proxy(iterator);
	}
	
	@Override
	protected String[] defaultReportColumns(DBTable table) {
		return appendColumnIfNotIncluded(expression, super.defaultReportColumns(table));
	}
	
	public long getDefectCount() {
		return defectCount;
	}

	public void reset() {
		defectCount = 0;
	}

	@Override
	public String toString() {
		return tableName + '.' + expression + " matches " + validator.toString();
	}

	public class Proxy extends TableRowIteratorProxy {
		
		private Object[] next;
		
		public Proxy(ArrayResultSetIterator source) {
			super(source);
			fetchNext();
		}

		public boolean hasNext() {
			return (next != null);
		}

		public Object[] next() {
			Object[] result = next;
			fetchNext();
			return result;
		}

		private void fetchNext() {
			Object[] tmp = null;
			boolean valid = true;
			while (super.hasNext() && valid) {
				tmp = super.next();
				Object columnValue = tmp[checkedColumnIndex];
				valid = validator.valid(columnValue);
			}
			if (!valid) {
				next = tmp;
				defectCount++;
			} else
				next = null;
		}

	}

}
